'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _ioredis = require('ioredis');

var _ioredis2 = _interopRequireDefault(_ioredis);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } } /*  strict
                                                                                                                                                           *
                                                                                                                                                           * TagCache
                                                                                                                                                           * Allows one to cache data with tags and invalidate based on them
                                                                                                                                                           *
                                                                                                                                                           * Under the hood this stores one set of keys per tag (ID) and the data per key in Redis:
                                                                                                                                                           * - `tags:asdf-123` = `db.table('threads').get('asdf-345') db.table('threads').get('asdf-234') ...`
                                                                                                                                                           * - `data:db.table('threads').get('asdf-345')` = `{ "id": "asdf-345", "content": { "title": "Hello" }, ... }`
                                                                                                                                                           */

var TagCache = function TagCache(options) {
  var _this = this;

  _classCallCheck(this, TagCache);

  this.get = function () {
    var _ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(key) {
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.prev = 0;
              return _context.abrupt('return', _this.redis.get('data:' + key).then(function (res) {
                try {
                  return JSON.parse(res);
                } catch (err) {
                  return res;
                }
              }));

            case 4:
              _context.prev = 4;
              _context.t0 = _context['catch'](0);
              return _context.abrupt('return', Promise.reject(_context.t0));

            case 7:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this, [[0, 4]]);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }();

  this.set = function () {
    var _ref2 = _asyncToGenerator(regeneratorRuntime.mark(function _callee2(key, data, tags) {
      var multi;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.prev = 0;
              _context2.next = 3;
              return _this.redis.multi();

            case 3:
              multi = _context2.sent;


              // Add the key to each of the tag sets
              tags.forEach(function (tag) {
                multi.sadd('tags:' + tag, key);
              });

              // Add the data to the key
              multi.set('data:' + key, JSON.stringify(data));
              _context2.next = 8;
              return multi.exec();

            case 8:
              return _context2.abrupt('return');

            case 11:
              _context2.prev = 11;
              _context2.t0 = _context2['catch'](0);
              return _context2.abrupt('return', Promise.reject(_context2.t0));

            case 14:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this, [[0, 11]]);
    }));

    return function (_x2, _x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }();

  this.invalidate = function () {
    var _ref3 = _asyncToGenerator(regeneratorRuntime.mark(function _callee3(tags) {
      var keys, pipeline;
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.prev = 0;
              _context3.t0 = [].concat;
              _context3.t1 = [];
              _context3.next = 5;
              return Promise.all(tags.map(function (tag) {
                return _this.redis.smembers('tags:' + tag);
              }));

            case 5:
              _context3.t2 = _context3.sent;
              keys = _context3.t0.apply.call(_context3.t0, _context3.t1, _context3.t2);
              _context3.next = 9;
              return _this.redis.pipeline();

            case 9:
              pipeline = _context3.sent;


              keys.forEach(function (key) {
                pipeline.del('data:' + key);
              });

              tags.forEach(function (tag) {
                pipeline.del('tags:' + tag);
              });

              _context3.next = 14;
              return pipeline.exec();

            case 14:
              _context3.next = 19;
              break;

            case 16:
              _context3.prev = 16;
              _context3.t3 = _context3['catch'](0);
              return _context3.abrupt('return', Promise.reject(_context3.t3));

            case 19:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, _this, [[0, 16]]);
    }));

    return function (_x5) {
      return _ref3.apply(this, arguments);
    };
  }();

  this.redis = new _ioredis2.default(options || {});
}

// How invalidation by tag works:
// 1. Get all the keys associated with all the passed-in tags (tags:${tag})
// 2. Delete all the keys data (data:${key})
// 3. Delete all the tags (tags:${tag})
;

exports.default = TagCache;